@@ -71,7 +71,10 @@ except ImportError:
         # For paths, just use path.name rather than converting with this
         # If we are not doing any cleverness with non-unicode filename bytes,
         # encoding to system encoding is good enough
-        return unicode_filename.encode(sys.getfilesystemencoding(), u"replace")
+        if sys.version_info[:2] >= (3, 2):
+            return os.fsencode(unicode_filename)
+        else:
+            return unicode_filename.encode(config.fsencoding, u"replace")
 
     def fsdecode(bytes_filename):
         u"""Convert a filename encoded in the system encoding to unicode"""
@@ -79,7 +82,10 @@ except ImportError:
         # If we are not doing any cleverness with non-unicode filename bytes,
         # decoding using system encoding is good enough. Use "ignore" as
         # Linux paths can contain non-Unicode characters
-        return bytes_filename.decode(config.fsencoding, u"replace")
+        if sys.version_info[:2] >= (3, 2):
+            return os.fsdecode(bytes_filename)
+        else:
+            return bytes_filename.decode(config.fsencoding, u"replace")
 
 
 def exception_traceback(limit=50):
@@ -288,30 +294,54 @@ def which(program):
 
 
 def start_debugger():
-    if not os.getenv(u'DEBUG_RUNNING', None) and (u'--pydevd' in sys.argv or os.getenv(u'PYDEVD', None)):
+    if u'--pydevd' in sys.argv or os.environ.get(u"PYDEVD", None):
         try:
-            import pydevd_pycharm as pydevd  # pylint: disable=import-error
+            import pydevd_pycharm  # pylint: disable=import-error
         except ImportError:
-            try:
-                import pydevd  # pylint: disable=import-error
-            except ImportError:
-                log.FatalError(u"Module pydevd_pycharm or pydevd must be available for debugging.\n"
-                               u"Remove '--pydevd' from command line and PYDEVD from environment\n"
-                               u"to avoid activating the debugger.")
+            log.FatalError(u"Module pydevd_pycharm must be available for debugging.\n"
+                           u"Remove '--pydevd' from command line and unset 'PYDEVD'\n"
+                           u"from the environment to avoid starting the debugger.")
 
+        # NOTE: this needs to be customized for your system
+        debug_host = u'dione.local'
+        debug_port = 6700
+
+        # get previous pid:port if any
+        # return if pid the same as ours
+        prev_port = None
+        debug_running = os.environ.get(u"DEBUG_RUNNING", False)
+        if debug_running:
+            prev_pid, prev_port = map(int, debug_running.split(u":"))
+            if prev_pid == os.getpid():
+                return
+
+        # new pid, next port, start a new debugger
+        if prev_port:
+            debug_port = int(prev_port) + 1
+
+        # ignition
         try:
-            # NOTE: this needs to be customized for your system
-            pydevd.settrace(u'dione.local', port=6777, stdoutToServer=True, stderrToServer=True)
+            pydevd_pycharm.settrace(debug_host,
+                                    port=debug_port,
+                                    suspend=False,
+                                    stdoutToServer=True,
+                                    stderrToServer=True,
+                                    # patch_multiprocessing=True,
+                                    )
+            log.Info(u"Connection {0}:{1} accepted for debug."
+                     .format(debug_host, debug_port))
         except ConnectionRefusedError as e:
-            log.FatalError(u"Connection refused for debug.  Check your setup.")
+            log.Info(u"Connection {0}:{1} refused for debug: {2}"
+                     .format(debug_host, debug_port, str(e)))
 
-        # In a dev environment the path is screwed so fix it.
+        # in a dev environment the path is screwed so fix it.
         base = sys.path.pop(0)
         base = base.split(os.path.sep)[:-1]
         base = os.path.sep.join(base)
         sys.path.insert(0, base)
 
-        os.environ[u'DEBUG_RUNNING'] = u'yes'
+        # save last debug pid:port used
+        os.environ[u'DEBUG_RUNNING'] = u"{0}:{1}".format(os.getpid(), debug_port)
 
 
 def merge_dicts(*dict_args):
